@@ -25,6 +25,7 @@
 #endif
 
 #include "pygobject-private.h"
+#include "pythread.h"
 
 static PyObject *gerror_exc = NULL;
 static const gchar *pyginterface_type_id   = "PyGInterface::type";
@@ -33,6 +34,8 @@ GQuark pyginterface_type_key  = 0;
 static void pyg_flags_add_constants(PyObject *module, GType flags_type,
 				    const gchar *strip_prefix);
 
+static gboolean use_gil_state_api = FALSE;
+    
 /* -------------- GDK threading hooks ---------------------------- */
 
 /**
@@ -950,11 +953,9 @@ pyg_type_register(PyObject *self, PyObject *args)
 	if (!PyDict_Check(gsignals)) {
 	    PyErr_SetString(PyExc_TypeError,
 			    "__gsignals__ attribute not a dict!");
-	    Py_DECREF(gsignals);
 	    return NULL;
 	}
 	if (!add_signals(instance_type, gsignals)) {
-	    Py_DECREF(gsignals);
 	    return NULL;
 	}
 	PyDict_DelItemString(class->tp_dict, "__gsignals__");
@@ -970,11 +971,9 @@ pyg_type_register(PyObject *self, PyObject *args)
 	if (!PyDict_Check(gproperties)) {
 	    PyErr_SetString(PyExc_TypeError,
 			    "__gproperties__ attribute not a dict!");
-	    Py_DECREF(gproperties);
 	    return NULL;
 	}
 	if (!add_properties(instance_type, gproperties)) {
-	    Py_DECREF(gproperties);
 	    return NULL;
 	}
 	PyDict_DelItemString(class->tp_dict, "__gproperties__");
@@ -1458,14 +1457,29 @@ pyg_main_context_default (PyObject *unused)
 
 }
 
+static int pyg_thread_state_tls_key = -1;
+
 static int
-pyg_enable_threads (void)
+pyg_enable_threads ()
 {
+    if (getenv ("PYGTK_USE_GIL_STATE_API"))
+	use_gil_state_api = TRUE;
+
 #ifndef DISABLE_THREADING
-    PyEval_InitThreads();
-    if (!g_threads_got_initialized)
-	g_thread_init(NULL);
-    pygobject_api_functions.threads_enabled = TRUE;
+    if (!pygobject_api_functions.threads_enabled) {
+	PyEval_InitThreads();
+	if (!g_threads_got_initialized)
+	    g_thread_init(NULL);
+	pygobject_api_functions.threads_enabled = TRUE;
+	pyg_thread_state_tls_key = PyThread_create_key();
+    }
+    if (PYGIL_API_IS_BUGGY && !use_gil_state_api) {
+	PyThreadState* state;
+	state = PyGILState_GetThisThreadState();
+	if ( state != NULL )
+	    PyThread_set_key_value(pyg_thread_state_tls_key, state);
+    }
+
     return 0;
 #else
     PyErr_SetString(PyExc_RuntimeError,
@@ -1474,11 +1488,62 @@ pyg_enable_threads (void)
 #endif
 }
 
+static PyThreadState *
+pyg_find_thread_state (void)
+{
+    PyThreadState* state;
+    
+    if (pyg_thread_state_tls_key == -1)
+	return NULL;
+    state = PyThread_get_key_value(pyg_thread_state_tls_key);
+    if (state == NULL) {
+	state = PyGILState_GetThisThreadState();
+	if (state != NULL)
+	    PyThread_set_key_value(pyg_thread_state_tls_key, state);
+    }
+    return state;
+}
+
+static int
+pyg_gil_state_ensure_py23 (void)
+{
+    if (PYGIL_API_IS_BUGGY && !use_gil_state_api) {
+	PyThreadState* state = pyg_find_thread_state();
+
+	if (state == NULL)
+	    return PyGILState_LOCKED;
+    
+	if (state == _PyThreadState_Current)
+	    return PyGILState_LOCKED;
+	else {
+	    PyEval_RestoreThread(state);
+	    return PyGILState_UNLOCKED;
+	}
+    } else {
+	return PyGILState_Ensure();
+    }
+}
+
+static void
+pyg_gil_state_release_py23 (int flag)
+{
+    if (PYGIL_API_IS_BUGGY && !use_gil_state_api) {
+	if (flag == PyGILState_UNLOCKED) {
+	    PyThreadState* state = pyg_find_thread_state();
+	    if (state != NULL)
+		PyEval_ReleaseThread(state);
+	}
+    } else {
+	PyGILState_Release(flag);
+    }
+}
+
 static PyObject *
-pyg_threads_init (PyObject *unused)
+pyg_threads_init (PyObject *unused, PyObject *args, PyObject *kwargs)
 {
     if (pyg_enable_threads())
         return NULL;
+    
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -1500,7 +1565,7 @@ static PyMethodDef pygobject_functions[] = {
     { "io_add_watch", (PyCFunction)pyg_io_add_watch, METH_VARARGS|METH_KEYWORDS },
     { "source_remove", pyg_source_remove, METH_VARARGS },
     { "main_context_default", (PyCFunction)pyg_main_context_default, METH_NOARGS },
-    { "threads_init", (PyCFunction)pyg_threads_init, METH_NOARGS },
+    { "threads_init", (PyCFunction)pyg_threads_init, METH_VARARGS|METH_KEYWORDS },
     { NULL, NULL, 0 }
 };
 
@@ -1815,7 +1880,9 @@ struct _PyGObject_Functions pygobject_api_functions = {
   pyg_flags_from_gtype,
 
   FALSE, /* threads_enabled */
-  pyg_enable_threads
+  pyg_enable_threads,
+  pyg_gil_state_ensure_py23,
+  pyg_gil_state_release_py23
 };
 
 #define REGISTER_TYPE(d, type, name) \
